package com.example.orcs.and.tactics;

//import java.util.HashMap;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.TreeMap;

import org.andengine.engine.camera.BoundCamera;
import org.andengine.engine.camera.hud.HUD;
import org.andengine.engine.camera.hud.controls.BaseOnScreenControl;
import org.andengine.engine.camera.hud.controls.BaseOnScreenControl.IOnScreenControlListener;
import org.andengine.engine.camera.hud.controls.DigitalOnScreenControl;
import org.andengine.engine.handler.IUpdateHandler;
import org.andengine.engine.handler.timer.ITimerCallback;
import org.andengine.engine.handler.timer.TimerHandler;
import org.andengine.engine.options.EngineOptions;
import org.andengine.engine.options.ScreenOrientation;
import org.andengine.engine.options.resolutionpolicy.RatioResolutionPolicy;
import org.andengine.entity.IEntity;
import org.andengine.entity.modifier.PathModifier;
import org.andengine.entity.modifier.PathModifier.IPathModifierListener;
import org.andengine.entity.modifier.PathModifier.Path;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.scene.background.Background;
import org.andengine.entity.sprite.ButtonSprite;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.entity.util.FPSLogger;
import org.andengine.extension.tmx.TMXLayer;
import org.andengine.extension.tmx.TMXLoader;
import org.andengine.extension.tmx.TMXTiledMap;
import org.andengine.extension.tmx.util.exception.TMXLoadException;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.font.FontFactory;
import org.andengine.opengl.texture.TextureOptions;
import org.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlas;
import org.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlasTextureRegionFactory;
import org.andengine.opengl.texture.region.ITextureRegion;
import org.andengine.ui.activity.SimpleBaseGameActivity;
import org.andengine.util.HorizontalAlign;
import org.andengine.util.algorithm.path.ICostFunction;
import org.andengine.util.algorithm.path.IPathFinder;
import org.andengine.util.algorithm.path.IPathFinderMap;
import org.andengine.util.algorithm.path.astar.AStarPathFinder;
import org.andengine.util.algorithm.path.astar.IAStarHeuristic;
import org.andengine.util.algorithm.path.astar.ManhattanHeuristic;
import org.andengine.util.color.Color;

import android.graphics.Typeface;
import android.opengl.GLES20;

public class OrcsAndTactics extends SimpleBaseGameActivity {
	//constants
	
	//engine
	private static int CAMERA_WIDTH = 480;
	private static int CAMERA_HEIGHT = 320;
	private BoundCamera mBoundChaseCamera;
	private Scene mScene;
	//tmx map
	private TMXTiledMap mTMXTiledMap;
	private TMXLayer mTMXLayer;
	
	//texture atlas, sprites, controller
	private BitmapTextureAtlas mBitmapTextureAtlas;
	private ITextureRegion mDwarfTextureRegion;
	private Player mDwarfPlayer;
	private ITextureRegion mOrcTextureRegion;
	private Player mOrcPlayer;
	private ITextureRegion mBlueButton;
	private ITextureRegion mRedButton;
	private ButtonSprite mBlueButtonSprite;
	private ButtonSprite mRedButtonSprite;
	private ITextureRegion mYellowButton;
	private ButtonSprite mYellowButtonSprite;
	private ITextureRegion mRedSquareTextureRegion;
	private Sprite mRedSquareSprite;
	private ITextureRegion mOnScreenControlBaseTextureRegion;
	private ITextureRegion mOnScreenControlKnobTextureRegion;
	private DigitalOnScreenControl mDigitalOnScreenControl;
	private ITextureRegion mRockTextureRegion;
	private Sprite rockSprite;
	
	
	//booleans
	private boolean playerControlEnabled = true;
	private boolean redButtonPressedOnce = false;
	private boolean targetHasBeenHit;
	private boolean yellowButtonPressedOnce = false;
	private boolean redSquareMotionEnabled = false;
	private boolean buttonsBlocked = false;
	private boolean isAbilityChosen = false;
	private boolean isEnemyTurn;
	private boolean playerHasMoved = false;
	//timerhandler
	private IUpdateHandler damageTextTimerHandler;
	
	//maps and lists
	//private HashMap <SpritePoint,Player> mCollisionMap;
	private ArrayList<Player> turnList;
	private Player actualPlayer;
	private int indexOfTurnList;
	private Adventurer targetAdventurer;
	private HashMap<SpritePoint, Sprite> collidingObjectsMap;
	//font
	private Font mFont;
	private Text damageText;
	private Text playerVisualizationText;
	
	//ints for the attackRoll and actions
	private int tempMovement;
	private int attackArmorDifference;
	private int damage;
	private int threshold;
	private int attackRoll;
	
	//floats
	private float posX;
	private float posY;
	private float nextPosX;
	private float nextPosY;
	private float nextX;
	private float nextY;
	private Float xdistance;
	private Float ydistance;
	
	private float layerHeight;
	private float layerWidth;
	//temporal vars
	private SpritePoint targetSpritePoint;
	private SpritePoint playerVisualizationSpritePoint;
	private SpritePoint temporalCollisionSpritePoint;
	private Ability actualAbilityChosen;
	private SpritePoint pointForCheckingAroundPlayer;
	private SpritePoint targetPlayerMoveTileForEnemy;
	private ArrayList<Ability> actualPlayerAbilities;
	
	private TimerHandler passTurnTimerHandler;
	
	public EngineOptions onCreateEngineOptions() {
		
		mBoundChaseCamera = new BoundCamera(0, 0, CAMERA_WIDTH, CAMERA_HEIGHT);
		
		return new EngineOptions(true, ScreenOrientation.LANDSCAPE_FIXED, 
				new RatioResolutionPolicy(CAMERA_WIDTH, CAMERA_HEIGHT), mBoundChaseCamera);
	}

	@Override
	protected void onCreateResources() {
		
		//player texture regions
		BitmapTextureAtlasTextureRegionFactory.setAssetBasePath("gfx/");
		mBitmapTextureAtlas = new BitmapTextureAtlas(this.getTextureManager(),512,512);
		mDwarfTextureRegion = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mBitmapTextureAtlas,
				this, "DwarfPixeled.png", 0, 0);
		mOrcTextureRegion = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mBitmapTextureAtlas,
				this, "orc.png", 32, 0);
		mRockTextureRegion = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mBitmapTextureAtlas,
				this, "rock.png", 64, 0);
		mBitmapTextureAtlas.load();
		
		//buttons and squares
		mYellowButton = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mBitmapTextureAtlas,
				this, "onscreen_control_buttonYellow.png", 0, 192);
		mBlueButton = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mBitmapTextureAtlas,
				this, "onscreen_control_buttonBlue.png", 0, 32);
		mRedButton = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mBitmapTextureAtlas,
				this, "onscreen_control_buttonRed.png", 0, 96);
		mRedSquareTextureRegion = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mBitmapTextureAtlas,
				this, "redSquare.png", 0, 160);

		//fonts
		this.mFont = FontFactory.create(this.getFontManager(), this.getTextureManager(),
				256, 256, Typeface.create(Typeface.DEFAULT, Typeface.NORMAL), 15);
		// TO - DO:
//		Color color = new Color(1f, 0.5f, 0);
//		this.mFont = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, Typeface.create(Typeface.DEFAULT,Typeface.NORMAL), 1f, color);
		this.mFont.load();
		//controller
		
		mOnScreenControlBaseTextureRegion = BitmapTextureAtlasTextureRegionFactory.createFromAsset
				(this.mBitmapTextureAtlas, this, "onscreen_control_base.png", 128, 0);
		mOnScreenControlKnobTextureRegion = BitmapTextureAtlasTextureRegionFactory.createFromAsset
				(this.mBitmapTextureAtlas, this, "onscreen_control_knob.png", 256, 0);
		
	}

	@Override
	protected Scene onCreateScene() {
		collidingObjectsMap = new HashMap<SpritePoint, Sprite>();
		indexOfTurnList = 0;
		targetSpritePoint = new SpritePoint();
		playerVisualizationSpritePoint = new SpritePoint();
		temporalCollisionSpritePoint = new SpritePoint();
		pointForCheckingAroundPlayer = new SpritePoint();
		targetPlayerMoveTileForEnemy = new SpritePoint();
		actualAbilityChosen = new Ability();
		
		this.mEngine.registerUpdateHandler(new FPSLogger());
		turnList = new ArrayList<Player>();
		mScene = new Scene();
		mScene.setBackground(new Background(0,0.5f,0.2f));
		
		damageText = new Text(0, 0, this.mFont, "default text",
				new TextOptions(HorizontalAlign.CENTER), this.getVertexBufferObjectManager());
		damageText.setVisible(false);
		
		playerVisualizationText = new Text(0, 0, this.mFont, "default text", 50, this.getVertexBufferObjectManager());
		playerVisualizationText.setVisible(false);
		
		//map loading
		try{
			final TMXLoader mTMXLoader = new TMXLoader(this.getAssets(), this.getTextureManager(),
					TextureOptions.BILINEAR_PREMULTIPLYALPHA, this.getVertexBufferObjectManager());
			this.mTMXTiledMap = mTMXLoader.loadFromAsset("tmx/field.tmx");
		}catch (final TMXLoadException e) {
			org.andengine.util.debug.Debug.e(e);
		}	
		mTMXLayer = mTMXTiledMap.getTMXLayers().get(0);
		mScene.attachChild(mTMXLayer);
		
		//layer width and height to create the borders of the map
		layerHeight = mTMXLayer.getHeight();
		layerWidth = mTMXLayer.getWidth();
		
		//create and attach the player sprite and red square
		mDwarfPlayer = new Dwarf(0, 0, mDwarfTextureRegion, this.getVertexBufferObjectManager());
		mScene.attachChild(mDwarfPlayer);
		mDwarfPlayer.setHitPoints(mDwarfPlayer.getMaxHitPoints());
		mDwarfPlayer.readyAbilities();
		mOrcPlayer = new Orc(64, 64, mOrcTextureRegion, this.getVertexBufferObjectManager());
		mScene.attachChild(mOrcPlayer);
		mOrcPlayer.setHitPoints(mOrcPlayer.getMaxHitPoints());
		mOrcPlayer.readyAbilities();
		
		rockSprite = new Sprite(32, 0, mRockTextureRegion, this.getVertexBufferObjectManager());
		mScene.attachChild(rockSprite);
		collidingObjectsMap.put(new SpritePoint(rockSprite.getX(), rockSprite.getY()),rockSprite);
		mRedSquareSprite = new Sprite(0, 0, mRedSquareTextureRegion, this.getVertexBufferObjectManager());
		mRedSquareSprite.setVisible(false);
		mScene.attachChild(mRedSquareSprite);
		
		mScene.attachChild(damageText);
		System.out.println(collidingObjectsMap.toString());
		//scene timer handler
		damageTextTimerHandler = new TimerHandler(1.5f, false, new ITimerCallback() {
			public void onTimePassed(final TimerHandler pTimerHandler) {
			damageText.setVisible(false);
			}
		});
		mScene.registerUpdateHandler(damageTextTimerHandler);
		
		final IPathModifierListener mIPathModifierListener = new IPathModifierListener() {
			
			public void onPathWaypointStarted(PathModifier pPathModifier,
					IEntity pEntity, int pWaypointIndex) {}
			public void onPathWaypointFinished(PathModifier pPathModifier,
					IEntity pEntity, int pWaypointIndex) {}
			public void onPathStarted(PathModifier pPathModifier, IEntity pEntity) {}
			public void onPathFinished(PathModifier pPathModifier, IEntity pEntity) {
				playerControlEnabled = true;
				redSquareMotionEnabled = true;
			}
		};
		
		//add sprites to collisionmap
		//mCollisionMap.put(new SpritePoint(mDwarfPlayer.getX(), mDwarfPlayer.getY()),mDwarfPlayer);
		//mCollisionMap.put(new SpritePoint(mOrcPlayer.getX(), mOrcPlayer.getY()), mOrcPlayer);
		turnList.add(mDwarfPlayer);	
		turnList.add(mOrcPlayer);
		actualPlayer = turnList.get(indexOfTurnList);
		if(!(actualPlayer instanceof Enemy)){
			isEnemyTurn = false;
		}else{
			isEnemyTurn = true;
		}
		mBoundChaseCamera.setChaseEntity(actualPlayer);
		tempMovement = actualPlayer.getMovement();
		
		this.mDigitalOnScreenControl = new DigitalOnScreenControl(0, CAMERA_HEIGHT - 
				this.mOnScreenControlBaseTextureRegion.getHeight(), this.mBoundChaseCamera, 
				this.mOnScreenControlBaseTextureRegion, this.mOnScreenControlKnobTextureRegion,
				0.1f, this.getVertexBufferObjectManager(), new IOnScreenControlListener() {
					
					public void onControlChange(BaseOnScreenControl pBaseOnScreenControl,
							float pValueX, float pValueY) {
						if((pValueX!=0 || pValueY!=0) && playerControlEnabled && !isEnemyTurn){
							posX = actualPlayer.getX();
							posY = actualPlayer.getY();
							nextPosX = posX+(pValueX*32);
							nextPosY = posY+(pValueY*32);
							temporalCollisionSpritePoint.setSpritePoint(nextPosX, nextPosY);
							if(retrievePlayerFromTurnList(turnList, temporalCollisionSpritePoint) == null && 
									nextPosX >=0 && nextPosX < layerWidth && nextPosY >=0
									&& nextPosY < layerHeight && tempMovement>0
									&& collidingObjectsMap.get(temporalCollisionSpritePoint)==null){
								playerControlEnabled = false;
								playerHasMoved = true;
								final Path path = new Path(2).to(posX,posY).to(nextPosX, nextPosY);
								actualPlayer.registerEntityModifier(new PathModifier(0.5f, path,null,mIPathModifierListener ));
								tempMovement--;
								playerVisualizationText.setText("move left: " + tempMovement);
								playerVisualizationText.setVisible(true);
							}
						}else if((pValueX!=0 || pValueY!=0) && !playerControlEnabled && redSquareMotionEnabled && !isEnemyTurn){
							nextX = mRedSquareSprite.getX()+32*pValueX;
							nextY = mRedSquareSprite.getY()+32*pValueY;
							float xright = actualPlayer.getX() + actualPlayer.getRange()*32;
							float xleft = actualPlayer.getX() - actualPlayer.getRange()*32;
							float ydown = actualPlayer.getY() + actualPlayer.getRange()*32;
							float yup = actualPlayer.getY() - actualPlayer.getRange()*32;
							float playerX = actualPlayer.getX();
							float playerY = actualPlayer.getY();
							if(nextX >=0 && nextX < layerWidth && nextY >=0 && nextY < layerHeight){
								if((nextX<xright && (nextY>yup && nextY<ydown)) &&
										(nextX>xleft && (nextY>yup && nextY<ydown)) || 
										((nextX<=xright && nextX>=xleft) && nextY==playerY) ||
										((nextY<=ydown && nextY>=yup) && nextX==playerX)
										){
									mRedSquareSprite.setPosition(nextX,nextY);
									playerVisualizationSpritePoint.setSpritePoint(nextX, nextY); 
									Player targetVisualizationPlayer = retrievePlayerFromTurnList(turnList, 
											playerVisualizationSpritePoint);	
									if(targetVisualizationPlayer!=null){
										playerVisualizationText.setText(targetVisualizationPlayer.getName() + 
												" HP: " + targetVisualizationPlayer.getHitPoints());
										playerVisualizationText.setVisible(true);
									}else if(targetVisualizationPlayer==null){
										playerVisualizationText.setVisible(false);
									}
								}
							}
						}
						
					}
				});
		this.mDigitalOnScreenControl.getControlBase().setBlendFunction(GLES20.GL_SRC_ALPHA, GLES20.GL_ONE_MINUS_SRC_ALPHA);
		this.mDigitalOnScreenControl.getControlBase().setAlpha(0.5f);
		this.mDigitalOnScreenControl.getControlBase().setScaleCenter(0, 64);
		this.mDigitalOnScreenControl.getControlBase().setScale(1.25f);
		this.mDigitalOnScreenControl.getControlKnob().setScale(1.25f);
		this.mDigitalOnScreenControl.refreshControlKnobPosition();
		mScene.setChildScene(this.mDigitalOnScreenControl);
		
		
		
		//sprite buttons:
		ButtonSprite.OnClickListener mBlueButtonClickListener = new ButtonSprite.OnClickListener() {
			
			public void onClick(ButtonSprite pButtonSprite, float pTouchAreaLocalX,
					float pTouchAreaLocalY) {
				if(redButtonPressedOnce && !isEnemyTurn){
					mRedSquareSprite.setVisible(false);
					playerControlEnabled = true;
					redButtonPressedOnce = false;
					playerVisualizationText.setVisible(false);
				}else if (!redButtonPressedOnce && !yellowButtonPressedOnce && !isEnemyTurn) {
					passTurn();
				}else if(isAbilityChosen && !isEnemyTurn){
					yellowButtonPressedOnce = false;
					isAbilityChosen = false;
					playerVisualizationText.setVisible(false);
				}
			}
		};
		//A star inits:
		
		
		//yellow button listener:
		ButtonSprite.OnClickListener mYellowButtonClickListener = new ButtonSprite.OnClickListener() {
			
			public void onClick(ButtonSprite pButtonSprite, float pTouchAreaLocalX,
					float pTouchAreaLocalY) {
				actualPlayerAbilities = actualPlayer.getPlayerAbilities();
				if(!yellowButtonPressedOnce && !isEnemyTurn && actualPlayerAbilities!=null){
					playerVisualizationText.setVisible(false);
					actualAbilityChosen = actualPlayerAbilities.get(0);
					yellowButtonPressedOnce = true;
					playerVisualizationText.setText("Choose an ability:\n  " 
							+ actualAbilityChosen.getAbilityName());
					playerVisualizationText.setVisible(true);
					isAbilityChosen = true;
				}else if(yellowButtonPressedOnce && !isEnemyTurn && actualPlayerAbilities!=null){
					playerVisualizationText.setVisible(false);
					actualAbilityChosen = actualPlayerAbilities.get(1);
					yellowButtonPressedOnce = false;
					playerVisualizationText.setText("Choose an ability:\n  " 
							+ actualAbilityChosen.getAbilityName());
					playerVisualizationText.setVisible(true);
					isAbilityChosen = true;
				}		
			}
		};	
		//red button listener:
		ButtonSprite.OnClickListener mRedButtonClickListener = new ButtonSprite.OnClickListener() {
			
			public void onClick(ButtonSprite pButtonSprite, float pTouchAreaLocalX,
					float pTouchAreaLocalY) {
				if(!redButtonPressedOnce && !isEnemyTurn && !isAbilityChosen){
					System.out.println("first if");
					playerControlEnabled = false;
					mRedSquareSprite.setPosition(actualPlayer.getX(), actualPlayer.getY());
					mRedSquareSprite.setVisible(true);
					redButtonPressedOnce = true;
					redSquareMotionEnabled = true;
				}
				else if(redButtonPressedOnce && !isEnemyTurn && !isAbilityChosen){
					System.out.println("second if");
					redSquareMotionEnabled = false;
					targetSpritePoint.setSpritePoint(mRedSquareSprite.getX(), mRedSquareSprite.getY());
					Player targetPlayer = retrievePlayerFromTurnList(turnList, targetSpritePoint);	
					if(targetPlayer!=null && actualAbilityChosen instanceof Charge && actualAbilityChosen.getIsActive()){
						SpritePoint tempSpritePointNextToTarget = new SpritePoint();
						float distanceX = targetPlayer.getX() - actualPlayer.getX();
						float distanceY = targetPlayer.getY() - actualPlayer.getY();
						tempSpritePointNextToTarget = checkSurroundingsOfPlayer(distanceX, distanceY, targetPlayer);
						if(tempSpritePointNextToTarget!=null){
							playerControlEnabled = false;
//							final Path path = new Path(2).to(actualPlayer.getX(),actualPlayer.getY()).
//									to(tempSpritePointNextToTarget.getX(), tempSpritePointNextToTarget.getY());
							Path path = getAStarPath(actualPlayer, tempSpritePointNextToTarget);
							actualPlayer.registerEntityModifier(new PathModifier(0.5f, path,null,mIPathModifierListener));
							attackRoll(actualPlayer,targetPlayer);
						}
					}else if(targetPlayer!=null){
						if(targetPlayer.getX()!=actualPlayer.getX() || 
								targetPlayer.getY()!=actualPlayer.getY()){
							attackRoll(actualPlayer,targetPlayer);
						}
					}
					playerControlEnabled = true;
					mRedSquareSprite.setVisible(false);
					playerVisualizationText.setVisible(false);
					redButtonPressedOnce = false;
				}else if(isAbilityChosen && !isEnemyTurn && !redButtonPressedOnce){
					System.out.println(actualAbilityChosen.getAbilityName());
					yellowButtonPressedOnce = false;
					isAbilityChosen = false;
					playerVisualizationText.setVisible(false);
					if(actualAbilityChosen instanceof PassiveAbility){
						actualAbilityChosen.setIsActive(true);
						((PassiveAbility) actualAbilityChosen).resetTurnCounter();
						System.out.println(actualPlayer.getArmor());
						passTurn();
					}else if(actualAbilityChosen instanceof ActiveAbility){
						actualAbilityChosen.setIsActive(true);
						isAbilityChosen = false;
					}
				}
			}			
		};
		
		HUD hud = new HUD();
		mBlueButtonSprite = new ButtonSprite(CAMERA_WIDTH - 132, CAMERA_HEIGHT-66, mBlueButton,
				this.getVertexBufferObjectManager(),mBlueButtonClickListener);
		hud.attachChild(mBlueButtonSprite);
		hud.registerTouchArea(mBlueButtonSprite);
		
		mRedButtonSprite = new ButtonSprite(CAMERA_WIDTH - 66,CAMERA_HEIGHT - 66 ,mRedButton,
				this.getVertexBufferObjectManager(),mRedButtonClickListener);
		hud.attachChild(mRedButtonSprite);
		hud.registerTouchArea(mRedButtonSprite);
		
		mYellowButtonSprite = new ButtonSprite(CAMERA_WIDTH - 99,CAMERA_HEIGHT - 110 ,mYellowButton,
				this.getVertexBufferObjectManager(),mYellowButtonClickListener);
		hud.attachChild(mYellowButtonSprite);
		hud.registerTouchArea(mYellowButtonSprite);
		
		playerVisualizationText.setX(188);
		playerVisualizationText.setY(CAMERA_HEIGHT-64);
		hud.attachChild(playerVisualizationText);
		
		this.mBoundChaseCamera.setHUD(hud);
		passTurnTimerHandler = new TimerHandler(1f, false, new ITimerCallback() {
			public void onTimePassed(TimerHandler pTimerHandler) {
				passTurn();	
			}
		});
		return mScene;
	}
	private void attackRoll(Player attacker, Player target) {
		attackArmorDifference = attacker.getAttack() - target.getArmor();
		if(attackArmorDifference == 0){
			threshold = 50;
		}else if(attackArmorDifference > 0){
			threshold = 25;
		}else{
			threshold = 75;
		}
		attackRoll = (int)(Math.random()*100);
		targetHasBeenHit = attackRoll >= threshold;
		System.out.println("attack roll " + attackRoll);
		if(targetHasBeenHit){
			damage = (int)(Math.random()*(attacker.getMaxDamage()-
					attacker.getMinDamage())+attacker.getMinDamage());
			target.setHitPoints(target.getHitPoints()-damage);
			damageText.setText("" + damage);
			damageText.setX(target.getX() + 10);
			damageText.setY(target.getY() - 20);
			damageText.setVisible(true);
			damageTextTimerHandler.reset();
			
		}else{
			damageText.setText("MISS!");
			damageText.setX(target.getX() - 2);
			damageText.setY(target.getY() - 20);
			damageText.setVisible(true);
			damageTextTimerHandler.reset();
		}
		passTurnTimerHandler.reset();
		mScene.registerUpdateHandler(passTurnTimerHandler);
//		passTurn();
	}
	public Player retrievePlayerFromTurnList(ArrayList<Player> tL, SpritePoint sp){
		int i = 0;
		boolean found = false;
		Player tP = null;
		Player targetPlayer = null;
		while(i<tL.size() && !found){
			tP = tL.get(i);
			if(tP.getX() == sp.getX() && tP.getY() == sp.getY()){
				targetPlayer = tP;
				found = true;
			}
			i++;
		}
		return targetPlayer;
	}
	public Collection<Adventurer> getAdventurersByProximity(
			ArrayList<Player> turnList, Player actualPlayer) {
		Player tempPlayer = null;
		TreeMap<Float,Adventurer> adventurerSortedMapByProximity = new TreeMap<Float, Adventurer>();
		Float tempKey;
		for(int i = 0; i<turnList.size(); i++){
			tempPlayer = turnList.get(i);
			if(tempPlayer instanceof Adventurer){
				Float xdist = actualPlayer.getX()-tempPlayer.getX();
				Float ydist = actualPlayer.getY()-tempPlayer.getY();
				tempKey = (float) Math.sqrt(Math.pow(xdist, 2)+Math.pow(ydist, 2));
				adventurerSortedMapByProximity.put(tempKey,(Adventurer) tempPlayer);
			}
		}
		return adventurerSortedMapByProximity.values();
	}
	public void passTurn(){
		
		if(indexOfTurnList == turnList.size()-1){
			indexOfTurnList = 0;
		}else{
			indexOfTurnList ++;
		}
		actualPlayer = turnList.get(indexOfTurnList);
		playerHasMoved = false;
		mBoundChaseCamera.setChaseEntity(actualPlayer);
		tempMovement = actualPlayer.getMovement();
		System.out.println(actualPlayer.getClass());
		if(actualPlayer.getPlayerAbilities()!=null){
			for(Ability a:actualPlayer.getPlayerAbilities()){
				if(a instanceof PassiveAbility && a.getIsActive()){
					if(((PassiveAbility) a).getTurnCounter()==0){
						a.setIsActive(false);
					}else{
					((PassiveAbility) a).turnCounterDown();
					}
				}
			}
		}
		if(actualPlayer instanceof Enemy){
			isEnemyTurn = true;
			System.out.println("enters if instanceof Enemy");
			ArrayList<Adventurer> listOfAdventurersSortedByProximity = new 
					ArrayList<Adventurer>(getAdventurersByProximity(turnList,actualPlayer));
			int i = 0;
			boolean adyacentToEnemy = false;
			while(i<listOfAdventurersSortedByProximity.size()&&!adyacentToEnemy){
				System.out.println("arrives at while with index: " + i);
				targetAdventurer = listOfAdventurersSortedByProximity.get(i);
				if((targetAdventurer.getX()==(actualPlayer.getX()+32)) &&(targetAdventurer.getY()==actualPlayer.getY())
						||(targetAdventurer.getX()==(actualPlayer.getX()-32)) && (targetAdventurer.getY()==actualPlayer.getY())
						||(targetAdventurer.getY()==(actualPlayer.getY()-32)) && (targetAdventurer.getX()==actualPlayer.getX())
						||(targetAdventurer.getY()==(actualPlayer.getY()+32)) && (targetAdventurer.getX()==actualPlayer.getX())){
							adyacentToEnemy = true;
							System.out.println("\nadyacent to enemy = " + adyacentToEnemy);
				}else{
					xdistance = targetAdventurer.getX() - actualPlayer.getX();
					ydistance = targetAdventurer.getY() - actualPlayer.getY();
					targetPlayerMoveTileForEnemy = checkSurroundingsOfPlayer(xdistance, ydistance, targetAdventurer);
					i++;
				}
			}
			if(adyacentToEnemy){
				System.out.println("\ntarget adyacent to enemy ");
				isEnemyTurn = false;
				attackRoll(actualPlayer, targetAdventurer);
			}else if(targetPlayerMoveTileForEnemy!=null){
				Path path = getAStarPath(actualPlayer,targetPlayerMoveTileForEnemy);
				actualPlayer.registerEntityModifier(new PathModifier(path.getLength()/64, path, new IPathModifierListener() {
					
					public void onPathWaypointStarted(PathModifier pPathModifier,
							IEntity pEntity, int pWaypointIndex) {
					}				
					public void onPathWaypointFinished(PathModifier pPathModifier,
							IEntity pEntity, int pWaypointIndex) {
					}
					public void onPathStarted(PathModifier pPathModifier, IEntity pEntity) {						
					}					
					public void onPathFinished(PathModifier pPathModifier, IEntity pEntity) {
						isEnemyTurn = false;
						attackRoll(actualPlayer, targetAdventurer);
					}
				}));
			}else if(targetPlayerMoveTileForEnemy==null){
				isEnemyTurn = false;
				passTurn();
			}
			
		}
		
	}
	public SpritePoint checkSurroundingsOfPlayer(float xdistance, float ydistance, Player tempAdventurer){
		System.out.println("\narrives at method checkSurroundingsOfPlayer with x and y: " +xdistance+","+ydistance);
		boolean tileBlocked = true;
		if(xdistance < 0 && ydistance < 0){
			//CASE UP-LEFT
			System.out.println("\narrives at case up left");
			pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX()+32,tempAdventurer.getY());
			tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
			System.out.println("\nfirst check tile blocked " + tileBlocked);
			if(tileBlocked){
				pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX(),tempAdventurer.getY()+32);
				tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
				System.out.println("\nsecond check tile blocked " + tileBlocked);
			}
		}else if(xdistance < 0 && ydistance > 0){
			//CASE DOWN-LEFT
			System.out.println("\narrives at case down left");
			pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX()+32,tempAdventurer.getY());
			tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
			if(tileBlocked){
				pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX(),tempAdventurer.getY()-32);
				tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
			}
		}else if(xdistance < 0 && ydistance== 0){
			//CASE LEFT
			System.out.println("\narrives at case left");
			pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX()+32,tempAdventurer.getY());
			tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
		}else if(xdistance > 0 && ydistance == 0){
			//CASE RIGHT
			System.out.println("\narrives at case right");
			pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX()-32,tempAdventurer.getY());
			tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
		}else if(xdistance == 0 && ydistance < 0){
			//CASE UP
			System.out.println("\narrives at case up");
			pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX(),tempAdventurer.getY()+32);
			tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
		}else if(xdistance == 0 && ydistance >0){
			//CASE DOWN
			System.out.println("\narrives at case down");
			pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX(),tempAdventurer.getY()-32);
			tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
		}else if(xdistance > 0 && ydistance<0){
			//CASE UP-RIGHT
			System.out.println("\narrives at case up right");
			pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX()-32,tempAdventurer.getY());
			tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
			if(tileBlocked){
				pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX(),tempAdventurer.getY()+32);
				tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
			}
		}else if(xdistance > 0 && ydistance > 0){
			//CASE DOWN-RIGHT
			System.out.println("\narrives at case down right");
			pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX()-32,tempAdventurer.getY());
			tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
			if(tileBlocked){
				pointForCheckingAroundPlayer.setSpritePoint(tempAdventurer.getX(),tempAdventurer.getY()-32);
				tileBlocked = checkTileBlocked(pointForCheckingAroundPlayer,tempAdventurer);
			}
		}
		if(tileBlocked){
			System.out.println("\nnot calculating the tile");
			return null;
		}else if(!tileBlocked){
			System.out.println("the algorithm succeeds in calculating the tile");
			return pointForCheckingAroundPlayer;
		}
		return null;
	}
	public boolean checkTileBlocked(SpritePoint spritePoint,Player targetAdventurer){
		boolean isBlocked = false;
		for(Player p: turnList){
			if(p.getX()!=targetAdventurer.getX()&&p.getY()!=targetAdventurer.getY()
					&& spritePoint.getX()==p.getX() && p.getY()==spritePoint.getY()){
				isBlocked = true;
				break;
			}
		}
		if(collidingObjectsMap.get(spritePoint)!=null){
			isBlocked = true;
		}
		return isBlocked;
	}
	public Path getAStarPath(final Player actualPlayer, SpritePoint targetPlayerMoveTileForEnemy){
		IAStarHeuristic<TMXLayer> heuristic = new ManhattanHeuristic<TMXLayer>();
		IPathFinder<TMXLayer> finder = new AStarPathFinder<TMXLayer>();
		final org.andengine.util.algorithm.path.Path astarPath = finder.findPath(new IPathFinderMap<TMXLayer>() {
			public boolean isBlocked(int pX, int pY, TMXLayer pEntity) {
				boolean blocked = false;
				int i = 0;
				while(!blocked &&  i<turnList.size()){
					if(turnList.get(i).getX()==pX && turnList.get(i).getY()==pY && pX!=actualPlayer.getX() &&
							pY!=actualPlayer.getY()){
						blocked = true;
					}
					i++;
				}
				if(collidingObjectsMap.get(new SpritePoint(pX,pY))!=null){
					blocked = true;
				}
				return blocked;
			}
		}, 0, 0, (int)layerWidth, (int)layerHeight, mTMXLayer, (int)actualPlayer.getX(), (int)actualPlayer.getY(),
		(int)targetPlayerMoveTileForEnemy.getX(), (int)targetPlayerMoveTileForEnemy.getY(), false, heuristic,
				new ICostFunction<TMXLayer>() {

					public float getCost(IPathFinderMap<TMXLayer> pPathFinderMap,
							int pFromX, int pFromY, int pToX, int pToY,
							TMXLayer pEntity) {
						return 0;
					}
				});
		
		final int astarPathLength = astarPath.getLength();
		Path tPath = new Path(astarPathLength);
		for(int i = 0; i < astarPathLength; i++){
			tPath.to(astarPath.getX(i),astarPath.getY(i));
		}
		return tPath;
	}
}